@@ -22,7 +22,7 @@
 #include "core_phloat.h"
 
 bool persist_math();
-bool unpersist_math(bool discard);
+bool unpersist_math(int ver, bool discard);
 void reset_math();
 
 void put_shadow(const char *name, int length, phloat value);
